@@ -8,7 +8,7 @@ use base qw(Exporter DynaLoader);
 use Carp;
 use Try::Tiny;
 
-our $VERSION = '0.12';
+our $VERSION = '0.13';
 
 sub dl_load_flags { 1 } # Make PerlXLib.c functions available to other XS modules
 
@@ -37,21 +37,32 @@ my %_constants= (
     PointerMotionMask PropertyChangeMask ResizeRedirectMask
     StructureNotifyMask SubstructureNotifyMask SubstructureRedirectMask
     VisibilityChangeMask )],
+  const_ext_composite => [qw( CompositeRedirectAutomatic
+    CompositeRedirectManual )],
+  const_ext_shape => [qw( ShapeBounding ShapeClip ShapeInput ShapeIntersect
+    ShapeInvert ShapeSet ShapeSubtract ShapeUnion )],
   const_input => [qw( AnyKey AnyModifier AsyncBoth AsyncKeyboard AsyncPointer
     Button1Mask Button2Mask Button3Mask Button4Mask Button5Mask ControlMask
     GrabModeAsync GrabModeSync LockMask Mod1Mask Mod2Mask Mod3Mask Mod4Mask
-    Mod5Mask NoSymbol ReplayKeyboard ReplayPointer ShiftMask SyncBoth
-    SyncKeyboard SyncPointer XK_VoidSymbol )],
+    Mod5Mask NoSymbol PointerRoot ReplayKeyboard ReplayPointer RevertToNone
+    RevertToParent RevertToPointerRoot ShiftMask SyncBoth SyncKeyboard
+    SyncPointer XK_VoidSymbol )],
   const_sizehint => [qw( PAspect PBaseSize PMaxSize PMinSize PPosition
     PResizeInc PSize PWinGravity USPosition USSize )],
   const_visual => [qw( VisualAllMask VisualBitsPerRGBMask VisualBlueMaskMask
     VisualClassMask VisualColormapSizeMask VisualDepthMask VisualGreenMaskMask
     VisualIDMask VisualRedMaskMask VisualScreenMask )],
-  const_win => [qw( CopyFromParent InputOnly InputOutput )],
+  const_win => [qw( Above Below BottomIf CenterGravity CopyFromParent
+    EastGravity ForgetGravity InputOnly InputOutput LowerHighest
+    NorthEastGravity NorthGravity NorthWestGravity Opposite RaiseLowest
+    SouthEastGravity SouthGravity SouthWestGravity StaticGravity TopIf
+    UnmapGravity WestGravity )],
   const_winattr => [qw( CWBackPixel CWBackPixmap CWBackingPixel CWBackingPlanes
-    CWBackingStore CWBitGravity CWBorderPixel CWBorderPixmap CWColormap
-    CWCursor CWDontPropagate CWEventMask CWOverrideRedirect CWSaveUnder
-    CWWinGravity )],
+    CWBackingStore CWBitGravity CWBorderPixel CWBorderPixmap CWBorderWidth
+    CWColormap CWCursor CWDontPropagate CWEventMask CWHeight
+    CWOverrideRedirect CWSaveUnder CWSibling CWStackMode CWWidth CWWinGravity
+    CWX CWY )],
+  const_x => [qw( None )],
 # END GENERATED XS CONSTANT LIST
 );
 my %_functions= (
@@ -59,11 +70,11 @@ my %_functions= (
   fn_conn => [qw( ConnectionNumber XCloseDisplay XOpenDisplay XServerVendor
     XSetCloseDownMode XVendorRelease )],
   fn_event => [qw( XCheckMaskEvent XCheckTypedEvent XCheckTypedWindowEvent
-    XCheckWindowEvent XFlush XNextEvent XPutBackEvent XSelectInput XSendEvent
-    XSync )],
+    XCheckWindowEvent XEventsQueued XFlush XNextEvent XPending XPutBackEvent
+    XQLength XSelectInput XSendEvent XSync )],
   fn_input => [qw( XAllowEvents XBell XGrabButton XGrabKey XGrabKeyboard
-    XGrabPointer XQueryKeymap XUngrabButton XUngrabKey XUngrabKeyboard
-    XUngrabPointer keyboard_leds )],
+    XGrabPointer XQueryKeymap XQueryPointer XSetInputFocus XUngrabButton
+    XUngrabKey XUngrabKeyboard XUngrabPointer keyboard_leds )],
   fn_keymap => [qw( XDisplayKeycodes XGetKeyboardMapping XGetModifierMapping
     XKeysymToKeycode XLookupString XRefreshKeyboardMapping XSetModifierMapping
     load_keymap save_keymap )],
@@ -75,11 +86,18 @@ my %_functions= (
   fn_screen => [qw( DefaultColormap DefaultDepth DefaultGC DefaultScreen
     DefaultVisual DisplayHeight DisplayHeightMM DisplayWidth DisplayWidthMM
     RootWindow ScreenCount )],
+  fn_thread => [qw( XInitThreads XLockDisplay XUnlockDisplay )],
   fn_vis => [qw( XCreateColormap XFreeColormap XGetVisualInfo XMatchVisualInfo
     XVisualIDFromVisual )],
-  fn_win => [qw( XCreateSimpleWindow XCreateWindow XDestroyWindow XGetGeometry
-    XGetWMNormalHints XGetWMSizeHints XMapWindow XSetWMNormalHints
-    XSetWMSizeHints XUnmapWindow )],
+  fn_win => [qw( XChangeWindowAttributes XCirculateSubwindows XConfigureWindow
+    XCreateSimpleWindow XCreateWindow XDefineCursor XDestroyWindow
+    XGetGeometry XGetWMNormalHints XGetWMSizeHints XGetWindowAttributes
+    XLowerWindow XMapWindow XMoveResizeWindow XMoveWindow XQueryTree
+    XRaiseWindow XReparentWindow XResizeWindow XRestackWindows
+    XSetWMNormalHints XSetWMSizeHints XSetWindowBackground
+    XSetWindowBackgroundPixmap XSetWindowBorder XSetWindowBorderPixmap
+    XSetWindowBorderWidth XSetWindowColormap XTranslateCoordinates
+    XUndefineCursor XUnmapWindow )],
   fn_xtest => [qw( XTestFakeButtonEvent XTestFakeKeyEvent XTestFakeMotionEvent
     )],
 # END GENERATED XS FUNCTION LIST
@@ -92,7 +110,7 @@ our %EXPORT_TAGS= (
     functions => [ map { @$_ } values %_functions ],
     all => \@EXPORT_OK,
 );
-our @EXPORT= @{ $EXPORT_TAGS{fn_keysym} };
+our @EXPORT= @{ $EXPORT_TAGS{fn_keysym} }; # backward compatibility
 
 # Used by XS.  In the spirit of letting perl users violate encapsulation
 #  as needed, the XS code exposes its globals to Perl.
@@ -297,6 +315,32 @@ exported, and you can grab them all with
 
   use X11::Xlib ':functions';
 
+=head2 THREADING FUNCTIONS
+
+=head3 XInitThreads
+
+Sets up Xlib in a thread-safe manner, which basically means wrapping each Xlib
+method with a mutex.  After this call, multiple threads may access the same
+Display connection without application-level synchronization.
+If used, this must be the first Xlib call in the whole program, which can be
+inconvenient if you don't know in advance which other modules you are using.
+While perl scripts are typically single-threaded, you might still require this
+if you call into other libraries that create their own threads and also access
+Xlib.
+
+Returns true if multithread initialization succeeded.  If it fails, you
+probably should abort.  (and then fix your program so that it is the first
+Xlib function called)
+
+=head3 XLockDisplay
+
+Assuming XInitThreads succeeded, this will lock the Xlib mutex so you can run
+multiple calls uninterrupted.
+
+=head3 XUnlockDisplay
+
+Release the lock taken by L</XLockDisplay>.
+
 =head2 CONNECTION FUNCTIONS
 
 =head3 XOpenDisplay
@@ -347,6 +391,28 @@ Most of these functions return an L</XEvent> by way of an "out" parameter that
 gets overwritten during the call, in the style of C.  The variable receiving the
 event does not need to be initialized.
 
+=head3 XQLength
+
+  my $count= XQLength($display);
+
+Return number of events already in the incoming queue, without trying to read
+more.
+
+=head3 XPending
+
+  my $count= XPending($display);
+
+Return number of events in incoming queue after performing a flush and a read.
+
+=head3 XEventsQueued
+
+  my $count= XEventsQueued($display, $mode);
+
+C<$mode> is one of QueuedAlready, QueuedAfterFlush, or QueuedAfterReading.
+QueuedAlready simply returns the queue size.  QueuedAfterReading performs a
+read, then returns the count.  QueuedAfterFlush performs a flush and a read,
+then returns the count.
+
 =head3 XNextEvent
 
   XNextEvent($display, my $event_return)
@@ -414,7 +480,9 @@ incoming event queue.
 
   XSelectInput($display, $window, $event_mask)
 
-Change the event mask for a window.
+Change the event mask for a window.  Note that event masks are B<per-client>,
+so one client can listen to a window with a different mask than a second
+client listening to the same window.
 
 =head2 SCREEN ATTRIBUTES
 
@@ -628,6 +696,8 @@ Since this function didn't have nearly enough parameters for the imaginations
 of the Xlib creators, they added the full L<X11::Xlib::XSetWindowAttributes> structure
 as a final argument.  But to save you the trouble of setting all I<those>
 fields, they added an C<$attr_mask> to indicate which fields you are using.
+Simply OR together the constants listed in that struct.  If C<$attr_mask> is
+zero, then C<\%attrs> may be C<undef>.
 
 The window is initially un-mapped (i.e. hidden).  See L</XMapWindow>
 
@@ -654,15 +724,131 @@ the parent window is also mapped.  The server sends back a MapNotify event if
 the Window event mask allows it, and if a variety of other conditions are met.
 It's really pretty complicated and you should read the offical docs.
 
+=head3 XUnmapWindow
+
+  XUnmapWindow($display, $window);
+
+Hide a window.
+
 =head3 XGetGeometry
 
-  my ($root, $x, $y, $width, $height, $border_width, $color_depth);
-  XGetGeometry($display, $drawable, $root, $x, $y, $width, $height, $border_width, $color_depth)
+  my ($root, $x, $y, $width, $height, $border_width, $color_depth)
+    = XGetGeometry($display, $drawable)
     or die "XGetGeometry failed";
-  # All vars declared above should now be defined
 
-This function loads the geometry of the window into the variables you supply.
-You may omit the ones you don't care about.
+=head3 XGetWindowAttributes
+
+  my $bool= XGetWindowAttributes($display, $window, $attrs_out);
+
+Populate $attrs_out, which should be an undefined variable or a buffer or an
+instance of L<X11::Xlib::XWindowAttributes>.  If it returns false,
+C<$attrs_out> remains unchanged.
+
+=head3 XChangeWindowAttributes
+
+  XChangeWindowAttributes($display, $window, $valuemask, \%XSetWindowAttributes)
+
+Apply one or more fields of the L<X11::Xlib::XSetWindowAttributes> struct to
+the specified window.  C<$valuemask> is a ORed combination of the flags listed
+for that struct.
+
+=head3 XSetWindowBackground
+
+  XSetWindowBackground($display, $window, $background_pixel)
+
+Set the background pixel color (integer) for the window.
+
+=head3 XSetWindowBackgroundPixmap
+
+  XSetWindowBackgroundPixmap($display, $window, $background_pixmap)
+
+=head3 XSetWindowBorder
+
+  XSetWindowBorder($display, $window, $border_pixel)
+
+=head3 XSetWindowBorderPixmap
+
+  XSetWindowBorderPixmap($display, $window, $border_pixmap)
+
+=head3 XSetWindowColormap
+
+  XSetWindowColormap($display, $window, $colormap)
+
+=head3 XDefineCursor
+
+  XDefineCursor($display, $window, $cursor)
+
+=head3 XUndefineCursor
+
+  XUndefineCursor($display, $window)
+
+=head3 XReparentWindow
+
+  XReparentWindow($display, $wnd, $new_parent, $x, $y);
+
+Unmap, change parent, and remap C<$wnd> to be a child of C<$parent>.
+The X and Y arguments set the new location of the window relative to the
+parent client space.
+
+=head3 XConfigureWindow
+
+  XConfigureWindow($display, $window, $mask, \%XWindowChanges);
+
+Set the size, position, border, and stacking order of a window.
+L<X11::Xlib::XWindowChanges> can be passed as an object or plain hashref.
+C<$mask> is an ORed combination of the constants listed in the documentation
+for that struct, indicating which fields have been initialized.
+
+=head3 XMoveWindow
+
+  XMoveWindow($display, $window, $x, $y);
+
+=head3 XResizeWindow
+
+  XMoveWindow($display, $window, $width, $height)
+
+=head3 XMoveResizeWindow
+
+  XModeResizeWindow($display, $window, $x, $y, $width, $height)
+
+=head3 XSetWindowBorderWidth
+
+  XSetWindowBorderWidth($display, $window, $border_width)
+
+=head3 XQueryTree
+
+  my ($root, $parent, @children)= XQueryTree($display, $window);
+
+Return windows related to C<$window>.  Child windows are returned in
+bottom-to-top stacking order.  Returns an empty list if it fails.
+
+=head3 XRaiseWindow
+
+  XRaiseWindow($display, $window);
+
+Move window to front of stacking order.
+
+=head3 XLowerWindow
+
+ XLowerWindow($display, $window);
+
+Move window to back of stacking order.
+
+=head3 XCirculateSubwindows
+
+  XCirculateSubwindows($display, $parent_window, $direction);
+
+For the child windows of the given window, either bring the back-most to the
+front (C<direction == RaiseLowest>), or the front-most to the back
+(C<direction == LowerHighest>).
+
+(Note: use this instead of XCirculateSubwindowsUp or XCirculateSubwindowsDown)
+
+=head3 XRestackWindows
+
+  XRestackWindows($display, \@windows);
+
+Reset the stacking order of the specified windows, from front to back.
 
 =head3 XGetWMNormalHints
 
@@ -686,12 +872,6 @@ L<X11::Xlib::XSizeHints>, or a hashref of its fields.  Note that the C<< ->flags
 member of this struct will be initialized for you if you pass a hashref, according
 to what fields exist in the hashref.
 
-=head3 XUnmapWindow
-
-  XUnmapWindow($display, $window);
-
-Hide a window.
-
 =head3 XDestroyWindow
 
   XDestroyWindow($display, $window);
@@ -701,7 +881,9 @@ Unmap and destroy a window.
 =head2 XTEST INPUT SIMULATION
 
 These methods create fake server-wide input events, useful for automated testing.
-They are available through the XTEST extension.
+They are available through the XTest extension.  Currently this extension is a
+mandatory requirement for installing this module, and so these functions are
+always available.
 
 Don't forget to call L</XFlush> after these methods, if you want the events to
 happen immediately.
@@ -831,6 +1013,22 @@ True for vendor-private key codes.
 
 =head2 INPUT FUNCTIONS
 
+=head3 XSetInputFocus
+
+  XSetInputFocus($display, $wnd_focus, $revert_to, $time);
+
+Change input focus and set last-focus-time if C<$time> is after the current
+last-focus-time and before the current time of the X server.
+
+C<$time> can be CurrentTime to use the X server's clock.
+
+C<$wnd_focus> can be None to discard all keyboard input until a new window is
+focused, or PointerRoot to actively track the root window of whatever screen
+the pointer moves to.
+
+Once the target window becomes un-viewable, the C<$revert_to> setting takes
+effect, and can be RevertToParent, RevertToPointerRoot, or RevertToNone.
+
 =head3 XQueryKeymap
 
   XQueryKeymap($display)
@@ -1017,6 +1215,59 @@ Return the key code corresponding to C<$keysym> in the current mapping.
 
 Make the X server emit a sound.
 
+=head2 EXTENSION XCOMPOSITE
+
+This is an optional extension.  If you have Xcomposite available when this
+module was installed, then the following functions will be available.
+None of these functions are exportable.
+
+=head3 XCompositeVersion
+
+  my $version_integer= X11::Xlib::XCompositeVersion()
+    if X11::Xlib->can('X11::Xlib::XCompositeVersion');
+
+=head3 XCompositeQueryExtension
+
+  my ($event_base, $error_base)= $display->XCompositeQueryExtension
+    if $display->can('XCompositeQueryExtension');
+
+=head3 XCompositeQueryVersion
+
+  my ($major, $minor)= $display->XCompositeQueryExtension
+    if $display->can('XCompositeQueryExtension');
+
+=head3 XCompositeRedirectWindow
+
+  $display->XCompositeRedirectWindow($window, $update);
+
+=head3 XCompositeRedirectSubwindows
+
+  $display->XCompositeRedirectSubwindows($window, $update);
+
+=head3 XCompositeUnredirectWindow
+
+  $display->XCompositeUnredirectWindow($window, $update);
+
+=head3 XCompositeUnredirectSubwindows
+
+  $display->XCompositeUnredirectSubwindows($window, $update);
+
+=head3 XCompositeCreateRegionFromBorderClip
+
+  my $XserverRegion= $display->XCompositeCreateRegionFromBorderClip($window);
+
+=head3 XCompositeNameWindowPixmap
+
+  my $pixmap= $display->XCompositeNameWindowPixmap($window);
+
+=head3 XCompositeGetOverlayWindow
+
+  my $window= $display->XCompositeGetOverlayWindow($window);
+
+=head3 XCompositeReleaseOverlayWindow
+
+  $display->XCompositeReleaseOverlayWindow($window);
+
 =head1 STRUCTURES
 
 Xlib has a lot of C B<struct>s.  Most of them do not have much "depth"
