@@ -4,6 +4,26 @@ use warnings;
 use Carp;
 use parent 'X11::Xlib::XID';
 
+sub width  { croak "read-only" if @_ > 1; $_[0]{width} }
+sub height { croak "read-only" if @_ > 1; $_[0]{height} }
+sub depth  { croak "read-only" if @_ > 1; $_[0]{depth} }
+
+sub get_w_h { croak "read-only" if @_ > 1; $_[0]{width}, $_[0]{height} }
+
+sub DESTROY {
+    my $self= shift;
+    $self->display->XFreePixmap($self->xid)
+        if $self->autofree && $self->xid;
+}
+
+1;
+
+__END__
+
+=head1 NAME
+
+X11::Xlib::Pixmap - XID wrapper for Pixmap
+
 =head1 DESCRIPTION
 
 Object representing a Pixmap remote X11 resource.
@@ -35,18 +55,20 @@ Color depth, in bits.
 
 Reutrn the width and height of the pixmap as a list
 
+=head1 AUTHOR
+
+Olivier Thauvin, E<lt>nanardon@nanardon.zarb.orgE<gt>
+
+Michael Conrad, E<lt>mike@nrdvana.netE<gt>
+
+=head1 COPYRIGHT AND LICENSE
+
+Copyright (C) 2009-2010 by Olivier Thauvin
+
+Copyright (C) 2017 by Michael Conrad
+
+This library is free software; you can redistribute it and/or modify
+it under the same terms as Perl itself, either Perl version 5.10.0 or,
+at your option, any later version of Perl 5 you may have available.
+
 =cut
-
-sub width  { croak "read-only" if @_ > 1; $_[0]{width} }
-sub height { croak "read-only" if @_ > 1; $_[0]{height} }
-sub depth  { croak "read-only" if @_ > 1; $_[0]{depth} }
-
-sub get_w_h { croak "read-only" if @_ > 1; $_[0]{width}, $_[0]{height} }
-
-sub DESTROY {
-    my $self= shift;
-    $self->display->XFreePixmap($self->xid)
-        if $self->autofree && $self->xid;
-}
-
-1;
